require 'git'
require 'open3'
require 'shellwords'
require 'octokit'

module Ruboty
  module Packman
    class GithubRepository
      attr_reader :org, :name

      def initialize(org, name)
        @org = org
        @name = name
      end

      def full_name
        "#{org}/#{name}"
      end

      def sync_directory
        "#{repos_dir}/#{full_name}"
      end

      def sync!
        if cloned?
          git = Git.open(sync_directory)
          git.checkout('master')
          git.pull
        else
          Git.clone("https://#{token}:x-oauth-basic@#{domain}/#{full_name}", sync_directory)
        end
      end

      def bundle_update!
        cmd = [
          "cd #{sync_directory.shellescape}",
          "bundle install --path vendor/bundle",
          "bundle update"
        ]
        out, err, status = Bundler.with_clean_env { Open3.capture3(cmd.join(' && ')) }
      end

      def push!
        git = Git.open(sync_directory)
        git.branch(branch_name).checkout
        git.commit_all('bundle update')
        git.push('origin', branch_name)
      end

      def pull_request!
        pr = Octokit::Client.new(api_endpoint: api_endpoint, access_token: token).create_pull_request(
          full_name,
          'master',
          branch_name,
          "bundle update #{Time.now.strftime('%Y-%m-%d %H:%M:%S')}",
          'Pull requested from [packman](https://github.com/kimromi/ruboty-packman).'
        )
        pr[:html_url]
      rescue Octokit::UnprocessableEntity => e
        puts e
      end

      def clean!
        FileUtils.rm_rf(sync_directory) if cloned?
      end

      private

      def repos_dir
        ENV['PACKMAN_REPOS_DIR'] || File.realpath("#{__dir__}/../../../repos")
      end

      def token
        ENV['GITHUB_TOKEN']
      end

      def domain
        ENV['GITHUB_DOMAIN'] || 'github.com'
      end

      def api_endpoint
        ENV['GITHUB_API_ENDPOINT'] || 'https://api.github.com'
      end

      def cloned?
        Dir.exist?(sync_directory) && Git.open(sync_directory)
      rescue ArgumentError
        FileUtils.rm_rf(sync_directory)
        false
      end

      def branch_name
        @branch_name ||= "bundle-update-#{Time.now.strftime('%Y%m%d%H%M%S')}"
      end
    end
  end
end
